@@ -96,9 +96,7 @@ thunar_progress_dialog_init (ThunarProgressDialog *dialog)
   gtk_window_set_title (GTK_WINDOW (dialog), _("File Operation Progress"));
   gtk_window_set_default_size (GTK_WINDOW (dialog), 450, 10);
   gtk_window_set_modal (GTK_WINDOW (dialog), FALSE);
-  gtk_window_set_transient_for (GTK_WINDOW (dialog), NULL);
   gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), FALSE);
-  gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
 
   g_signal_connect (dialog, "delete-event",
                     G_CALLBACK (thunar_progress_dialog_closed), dialog);
@@ -204,7 +202,7 @@ thunar_progress_dialog_launch_view (ThunarProgressDialog *dialog,
       g_value_init (&title, G_TYPE_STRING);
       g_object_get_property (G_OBJECT (view), "title", &title);
       g_info ("Job \"%s\" does not exist in waiting list",
-              g_value_get_string (&title));
+              (gchar *) g_value_get_string (&title));
     }
 }
 
@@ -272,11 +270,7 @@ thunar_progress_dialog_job_finished (ThunarProgressDialog *dialog,
   if (n_views == SCROLLVIEW_THRESHOLD-1)
     {
       /* reparent the content box */
-#if LIBXFCE4UI_CHECK_VERSION (4, 13, 2)
       xfce_widget_reparent (dialog->content_box, dialog->vbox);
-#else
-      gtk_widget_reparent (dialog->content_box, dialog->vbox);
-#endif
 
       /* destroy the scroll win */
       gtk_widget_destroy (dialog->scrollwin);
@@ -403,11 +397,8 @@ thunar_progress_dialog_add_job (ThunarProgressDialog *dialog,
       gtk_widget_show (viewport);
 
       /* move the content box into the viewport */
-#if LIBXFCE4UI_CHECK_VERSION (4, 13, 2)
       xfce_widget_reparent (dialog->content_box, viewport);
-#else
-      gtk_widget_reparent (dialog->content_box, viewport);
-#endif
+
     }
 
   g_signal_connect_swapped (view, "need-attention",
