@@ -30,6 +30,7 @@
 #include <thunar/thunar-io-scan-directory.h>
 #include <thunar/thunar-io-jobs-util.h>
 #include <thunar/thunar-job.h>
+#include <thunar/thunar-job-operation-history.h>
 #include <thunar/thunar-preferences.h>
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-thumbnail-cache.h>
@@ -38,7 +39,7 @@
 
 
 /* seconds before we show the transfer rate + remaining time */
-#define MINIMUM_TRANSFER_TIME (10 * G_USEC_PER_SEC) /* 10 seconds */
+#define MINIMUM_TRANSFER_TIME (2 * G_USEC_PER_SEC) /* 2 seconds */
 
 
 
@@ -456,11 +457,12 @@ thunar_transfer_job_collect_node (ThunarTransferJob  *job,
 
 
 static gboolean
-ttj_copy_file (ThunarTransferJob *job,
-               GFile             *source_file,
-               GFile             *target_file,
-               GFileCopyFlags     copy_flags,
-               GError           **error)
+ttj_copy_file (ThunarTransferJob  *job,
+               ThunarJobOperation *operation,
+               GFile              *source_file,
+               GFile              *target_file,
+               GFileCopyFlags      copy_flags,
+               GError            **error)
 {
   GFileInfo *info;
   GFileType  source_type;
@@ -468,6 +470,7 @@ ttj_copy_file (ThunarTransferJob *job,
   gboolean   target_exists;
   gboolean   use_partial;
   gboolean   verify_file;
+  gboolean   add_to_operation = TRUE;
   GError    *err = NULL;
 
   _thunar_return_val_if_fail (THUNAR_IS_TRANSFER_JOB (job), FALSE);
@@ -575,6 +578,18 @@ ttj_copy_file (ThunarTransferJob *job,
       g_clear_object (&info);
     }
 
+  /* Preserve trusted state when launchers are copied */
+  if (G_UNLIKELY (err == NULL && g_file_is_native (source_file) && thunar_g_file_is_desktop_file (source_file)))
+    {
+      if (xfce_g_file_is_trusted (source_file, NULL, NULL) || thunar_g_file_is_in_xdg_data_dir (source_file))
+        {
+          xfce_g_file_set_trusted (target_file, TRUE, NULL, NULL);
+
+          /* launchers located in the XDG_DATA_DIR often dont have a +x flag */
+          thunar_g_file_set_executable_flags (target_file, NULL);
+        }
+    }
+
   /* check if there were errors */
   if (G_UNLIKELY (err != NULL && err->domain == G_IO_ERROR))
     {
@@ -586,6 +601,10 @@ ttj_copy_file (ThunarTransferJob *job,
           /* we tried to overwrite a directory with a directory. this normally results
            * in a merge. ignore that error, since we actually *want* to merge */
           g_clear_error (&err);
+
+          /* in case of a merge, do not register the directory, its descendants
+           * will be registered because of the recursion either way */
+          add_to_operation = FALSE;
         }
       else if (err->code == G_IO_ERROR_WOULD_RECURSE)
         {
@@ -613,6 +632,7 @@ ttj_copy_file (ThunarTransferJob *job,
               if (target_exists)
                 {
                   /* the target still exists and thus is not a directory. try to remove it */
+                  add_to_operation = TRUE;
                   g_file_delete (target_file,
                                  exo_job_get_cancellable (EXO_JOB (job)),
                                  &err);
@@ -637,6 +657,14 @@ ttj_copy_file (ThunarTransferJob *job,
     }
   else
     {
+      if (add_to_operation)
+        {
+          if (copy_flags & G_FILE_COPY_OVERWRITE)
+            thunar_job_operation_overwrite (operation, target_file);
+
+          if (thunar_job_get_log_mode (THUNAR_JOB (job)) == THUNAR_OPERATION_LOG_OPERATIONS)
+            thunar_job_operation_add (operation, source_file, target_file);
+        }
       return TRUE;
     }
 }
@@ -670,12 +698,13 @@ ttj_copy_file (ThunarTransferJob *job,
  *               on error or cancellation.
  **/
 static GFile *
-thunar_transfer_job_copy_file (ThunarTransferJob *job,
-                               GFile             *source_file,
-                               GFile             *target_file,
-                               gboolean           replace_confirmed,
-                               gboolean           rename_confirmed,
-                               GError           **error)
+thunar_transfer_job_copy_file (ThunarTransferJob     *job,
+                               ThunarJobOperation    *operation,
+                               GFile                 *source_file,
+                               GFile                 *target_file,
+                               gboolean               replace_confirmed,
+                               gboolean               rename_confirmed,
+                               GError               **error)
 {
   ThunarJobResponse response;
   GFile            *dest_file = target_file;
@@ -707,7 +736,7 @@ thunar_transfer_job_copy_file (ThunarTransferJob *job,
       if (err == NULL)
         {
           /* try to copy the file from source file to the duplicate file */
-          if (ttj_copy_file (job, source_file, target, copy_flags, &err))
+          if (ttj_copy_file (job, operation, source_file, target, copy_flags, &err))
             return target;
           else /* go to error case */
             g_object_unref (target);
@@ -734,6 +763,7 @@ thunar_transfer_job_copy_file (ThunarTransferJob *job,
           /* add overwrite flag and retry if we should overwrite */
           if (response == THUNAR_JOB_RESPONSE_REPLACE)
             {
+              replace_confirmed = TRUE;
               copy_flags |= G_FILE_COPY_OVERWRITE;
               continue;
             }
@@ -773,6 +803,7 @@ thunar_transfer_job_copy_file (ThunarTransferJob *job,
 
 static void
 thunar_transfer_job_copy_node (ThunarTransferJob  *job,
+                               ThunarJobOperation *operation,
                                ThunarTransferNode *node,
                                GFile              *target_file,
                                GFile              *target_parent_file,
@@ -920,7 +951,8 @@ retry_copy:
       thunar_transfer_job_check_pause (job);
 
       /* copy the item specified by this node (not recursively) */
-      real_target_file = thunar_transfer_job_copy_file (job, node->source_file,
+      real_target_file = thunar_transfer_job_copy_file (job, operation,
+                                                        node->source_file,
                                                         target_file,
                                                         node->replace_confirmed,
                                                         node->rename_confirmed,
@@ -939,7 +971,7 @@ retry_copy:
               if (node->children != NULL)
                 {
                   /* copy all children of this node */
-                  thunar_transfer_job_copy_node (job, node->children, NULL, real_target_file, NULL, &err);
+                  thunar_transfer_job_copy_node (job, operation, node->children, NULL, real_target_file, NULL, &err);
 
                   /* free resources allocted for the children */
                   thunar_transfer_node_free (node->children);
@@ -1212,6 +1244,7 @@ thunar_transfer_job_prepare_untrash_file (ExoJob     *job,
 
 static gboolean
 thunar_transfer_job_move_file_with_rename (ExoJob             *job,
+                                           ThunarJobOperation *operation,
                                            ThunarTransferNode *node,
                                            GList              *tp,
                                            GFileCopyFlags      flags,
@@ -1243,6 +1276,10 @@ thunar_transfer_job_move_file_with_rename (ExoJob             *job,
       if (!move_rename_successful && !exo_job_is_cancelled (job) && ((*error)->code == G_IO_ERROR_EXISTS))
         continue;
 
+      /* Log the operation if the move and rename were successful and logging is enabled */
+      if (thunar_job_get_log_mode (THUNAR_JOB (job)) == THUNAR_OPERATION_LOG_OPERATIONS)
+        thunar_job_operation_add (operation, node->source_file, renamed_file);
+
       return move_rename_successful;
     }
 }
@@ -1250,6 +1287,7 @@ thunar_transfer_job_move_file_with_rename (ExoJob             *job,
 
 static gboolean
 thunar_transfer_job_move_file (ExoJob                *job,
+                               ThunarJobOperation    *operation,
                                GFileInfo             *info,
                                GList                 *sp,
                                ThunarTransferNode    *node,
@@ -1287,11 +1325,18 @@ thunar_transfer_job_move_file (ExoJob                *job,
                                          move_flags | G_FILE_COPY_OVERWRITE,
                                          exo_job_get_cancellable (job),
                                          NULL, NULL, error);
+
+          if (move_successful && thunar_job_get_log_mode (THUNAR_JOB (job)) == THUNAR_OPERATION_LOG_OPERATIONS)
+            {
+              thunar_job_operation_overwrite (operation, tp->data);
+              thunar_job_operation_add (operation, node->source_file, tp->data);
+            }
+
         }
       /* if the user chose to rename then try to do so */
       else if (response == THUNAR_JOB_RESPONSE_RENAME)
         {
-          move_successful = thunar_transfer_job_move_file_with_rename (job, node, tp, move_flags, error);
+          move_successful = thunar_transfer_job_move_file_with_rename (job, operation, node, tp, move_flags, error);
         }
       /* if the user chose to cancel then abort all remaining file moves */
       else if (response == THUNAR_JOB_RESPONSE_CANCEL)
@@ -1308,6 +1353,12 @@ thunar_transfer_job_move_file (ExoJob                *job,
        * files will be released and the matching list items will be dropped below
        */
     }
+  else
+    {
+      if (thunar_job_get_log_mode (THUNAR_JOB (job)) == THUNAR_OPERATION_LOG_OPERATIONS)
+        thunar_job_operation_add (operation, node->source_file, tp->data);
+    }
+
   if (*error == NULL)
     {
       if (move_successful)
@@ -1549,6 +1600,7 @@ thunar_transfer_job_execute (ExoJob  *job,
   ThunarTransferNode   *node;
   ThunarApplication    *application;
   ThunarTransferJob    *transfer_job = THUNAR_TRANSFER_JOB (job);
+  ThunarJobOperation   *operation = NULL;
   GFileInfo            *info;
   GError               *err = NULL;
   GList                *new_files_list = NULL;
@@ -1556,6 +1608,7 @@ thunar_transfer_job_execute (ExoJob  *job,
   GList                *sp;
   GList                *tnext;
   GList                *tp;
+  gboolean              log_operations;
 
   _thunar_return_val_if_fail (THUNAR_IS_TRANSFER_JOB (job), FALSE);
   _thunar_return_val_if_fail (error == NULL || *error == NULL, FALSE);
@@ -1570,6 +1623,15 @@ thunar_transfer_job_execute (ExoJob  *job,
   thumbnail_cache = thunar_application_get_thumbnail_cache (application);
   g_object_unref (application);
 
+  /* whether or not we want to log operations to the undo list */
+  if (thunar_job_get_log_mode (THUNAR_JOB (transfer_job)) == THUNAR_OPERATION_LOG_OPERATIONS)
+    log_operations = TRUE;
+  else
+    log_operations = FALSE;
+
+  if (log_operations && transfer_job->type == THUNAR_TRANSFER_JOB_MOVE)
+    operation = thunar_job_operation_new (THUNAR_JOB_OPERATION_KIND_MOVE);
+
   for (sp = transfer_job->source_node_list, tp = transfer_job->target_file_list;
        sp != NULL && tp != NULL && err == NULL;
        sp = snext, tp = tnext)
@@ -1598,14 +1660,14 @@ thunar_transfer_job_execute (ExoJob  *job,
         {
           if (!thunar_transfer_job_prepare_untrash_file (job, info, tp->data, &err))
             break;
-          if (!thunar_transfer_job_move_file (job, info, sp, node, tp,
+          if (!thunar_transfer_job_move_file (job, operation, info, sp, node, tp,
                                               G_FILE_COPY_NOFOLLOW_SYMLINKS | G_FILE_COPY_ALL_METADATA,
                                               thumbnail_cache, &new_files_list, &err))
             break;
         }
       else if (transfer_job->type == THUNAR_TRANSFER_JOB_MOVE)
         {
-          if (!thunar_transfer_job_move_file (job, info, sp, node, tp,
+          if (!thunar_transfer_job_move_file (job, operation, info, sp, node, tp,
                                               G_FILE_COPY_NOFOLLOW_SYMLINKS | G_FILE_COPY_NO_FALLBACK_FOR_MOVE | G_FILE_COPY_ALL_METADATA,
                                               thumbnail_cache, &new_files_list, &err))
             break;
@@ -1643,12 +1705,15 @@ thunar_transfer_job_execute (ExoJob  *job,
       /* transfer starts now */
       transfer_job->start_time = g_get_real_time ();
 
+      if (log_operations && transfer_job->type == THUNAR_TRANSFER_JOB_COPY)
+        operation = thunar_job_operation_new (THUNAR_JOB_OPERATION_KIND_COPY);
+
       /* perform the copy recursively for all source transfer nodes */
       for (sp = transfer_job->source_node_list, tp = transfer_job->target_file_list;
            sp != NULL && tp != NULL && err == NULL;
            sp = sp->next, tp = tp->next)
         {
-          thunar_transfer_job_copy_node (transfer_job, sp->data, tp->data, NULL,
+          thunar_transfer_job_copy_node (transfer_job, operation, sp->data, tp->data, NULL,
                                          &new_files_list, &err);
         }
     }
@@ -1663,6 +1728,15 @@ thunar_transfer_job_execute (ExoJob  *job,
     {
       thunar_job_new_files (THUNAR_JOB (job), new_files_list);
       thunar_g_list_free_full (new_files_list);
+
+      /* Note that we only created a new thunar job operation of the appropriate kind (move or copy)
+       * in a mutually exclusive way, so we know that only one operation was created. */
+      if (log_operations)
+        {
+          thunar_job_operation_history_commit (operation);
+          g_object_unref (operation);
+        }
+
       return TRUE;
     }
 }
